@@ -21,12 +21,15 @@ import (
 	"encoding/base64"
 	"encoding/json"
 	"fmt"
+	"io"
 	"reflect"
+	"regexp"
 	"strings"
 	"testing"
 
 	"github.com/google/uuid"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"go.mongodb.org/mongo-driver/bson"
 )
 
@@ -174,56 +177,131 @@ func TestFormatMAC(t *testing.T) {
 func TestFormatUUID3(t *testing.T) {
 	first3 := uuid.NewMD5(uuid.NameSpaceURL, []byte("somewhere.com"))
 	other3 := uuid.NewMD5(uuid.NameSpaceURL, []byte("somewhereelse.com"))
+	other4 := uuid.Must(uuid.NewRandom())
+	other5 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhereelse.com"))
 	uuid3 := UUID3(first3.String())
 	str := other3.String()
-	testStringFormat(t, &uuid3, "uuid3", str, []string{}, []string{"not-a-uuid"})
+	testStringFormat(t, &uuid3, "uuid3", str,
+		[]string{
+			other3.String(),
+			strings.ReplaceAll(other3.String(), "-", ""),
+		},
+		[]string{
+			"not-a-uuid",
+			other4.String(),
+			other5.String(),
+			strings.ReplaceAll(other4.String(), "-", ""),
+			strings.ReplaceAll(other5.String(), "-", ""),
+			strings.Replace(other3.String(), "-", "", 2),
+			strings.Replace(other4.String(), "-", "", 2),
+			strings.Replace(other5.String(), "-", "", 2),
+		},
+	)
 
 	// special case for zero UUID
 	var uuidZero UUID3
 	err := uuidZero.UnmarshalJSON([]byte(jsonNull))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, UUID3(""), uuidZero)
 }
 
 func TestFormatUUID4(t *testing.T) {
 	first4 := uuid.Must(uuid.NewRandom())
+	other3 := uuid.NewMD5(uuid.NameSpaceURL, []byte("somewhere.com"))
 	other4 := uuid.Must(uuid.NewRandom())
+	other5 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhereelse.com"))
 	uuid4 := UUID4(first4.String())
 	str := other4.String()
-	testStringFormat(t, &uuid4, "uuid4", str, []string{}, []string{"not-a-uuid"})
+	testStringFormat(t, &uuid4, "uuid4", str,
+		[]string{
+			other4.String(),
+			strings.ReplaceAll(other4.String(), "-", ""),
+		},
+		[]string{
+			"not-a-uuid",
+			other3.String(),
+			other5.String(),
+			strings.ReplaceAll(other3.String(), "-", ""),
+			strings.ReplaceAll(other5.String(), "-", ""),
+			strings.Replace(other3.String(), "-", "", 2),
+			strings.Replace(other4.String(), "-", "", 2),
+			strings.Replace(other5.String(), "-", "", 2),
+		},
+	)
 
 	// special case for zero UUID
 	var uuidZero UUID4
 	err := uuidZero.UnmarshalJSON([]byte(jsonNull))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, UUID4(""), uuidZero)
 }
 
 func TestFormatUUID5(t *testing.T) {
 	first5 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhere.com"))
+	other3 := uuid.NewMD5(uuid.NameSpaceURL, []byte("somewhere.com"))
+	other4 := uuid.Must(uuid.NewRandom())
 	other5 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhereelse.com"))
 	uuid5 := UUID5(first5.String())
 	str := other5.String()
-	testStringFormat(t, &uuid5, "uuid5", str, []string{}, []string{"not-a-uuid"})
+	testStringFormat(t, &uuid5, "uuid5", str,
+		[]string{
+			other5.String(),
+			strings.ReplaceAll(other5.String(), "-", ""),
+		},
+		[]string{
+			"not-a-uuid",
+			other3.String(),
+			other4.String(),
+			strings.ReplaceAll(other3.String(), "-", ""),
+			strings.ReplaceAll(other4.String(), "-", ""),
+			strings.Replace(other3.String(), "-", "", 2),
+			strings.Replace(other4.String(), "-", "", 2),
+			strings.Replace(other5.String(), "-", "", 2),
+		},
+	)
 
 	// special case for zero UUID
 	var uuidZero UUID5
 	err := uuidZero.UnmarshalJSON([]byte(jsonNull))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, UUID5(""), uuidZero)
 }
 
 func TestFormatUUID(t *testing.T) {
 	first5 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhere.com"))
+	other3 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhereelse.com"))
+	other4 := uuid.Must(uuid.NewRandom())
 	other5 := uuid.NewSHA1(uuid.NameSpaceURL, []byte("somewhereelse.com"))
+	other6 := uuid.Must(uuid.NewV6())
+	other7 := uuid.Must(uuid.NewV7())
+	microsoft := "0" + other4.String() + "f"
+
 	uuid := UUID(first5.String())
 	str := other5.String()
-	testStringFormat(t, &uuid, "uuid", str, []string{}, []string{"not-a-uuid"})
+	testStringFormat(t, &uuid, "uuid", str,
+		[]string{
+			other3.String(),
+			other4.String(),
+			other5.String(),
+			strings.ReplaceAll(other3.String(), "-", ""),
+			strings.ReplaceAll(other4.String(), "-", ""),
+			strings.ReplaceAll(other5.String(), "-", ""),
+			other6.String(),
+			other7.String(),
+			microsoft,
+		},
+		[]string{
+			"not-a-uuid",
+			strings.Replace(other3.String(), "-", "", 2),
+			strings.Replace(other4.String(), "-", "", 2),
+			strings.Replace(other5.String(), "-", "", 2),
+		},
+	)
 
 	// special case for zero UUID
 	var uuidZero UUID
 	err := uuidZero.UnmarshalJSON([]byte(jsonNull))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, UUID(""), uuidZero)
 }
 
@@ -283,28 +361,28 @@ func TestFormatBase64(t *testing.T) {
 
 	var subj Base64
 	err := subj.UnmarshalText([]byte(str))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, expected, subj)
 
 	b, err = subj.MarshalText()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, []byte(str), b)
 
 	var subj2 Base64
 	err = subj2.UnmarshalJSON(bj)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, expected, subj2)
 
 	b, err = subj2.MarshalJSON()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, bj, b)
 
 	bsonData, err := bson.Marshal(subj2)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	var b64Copy Base64
 	err = bson.Unmarshal(bsonData, &b64Copy)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, subj2, b64Copy)
 
 	testValid(t, "byte", str)
@@ -312,7 +390,7 @@ func TestFormatBase64(t *testing.T) {
 
 	// Valuer interface
 	sqlvalue, err := subj2.Value()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	sqlvalueAsString, ok := sqlvalue.(string)
 	if assert.Truef(t, ok, "[%s]Value: expected driver value to be a string", "byte") {
 		assert.EqualValuesf(t, str, sqlvalueAsString, "[%s]Value: expected %v and %v to be equal", "byte", sqlvalue, str)
@@ -320,16 +398,16 @@ func TestFormatBase64(t *testing.T) {
 	// Scanner interface
 	var subj3 Base64
 	err = subj3.Scan([]byte(str))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, str, subj3.String())
 
 	var subj4 Base64
 	err = subj4.Scan(str)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, str, subj4.String())
 
 	err = subj4.Scan(123)
-	assert.Error(t, err)
+	require.Error(t, err)
 }
 
 type testableFormat interface {
@@ -350,14 +428,14 @@ func testStringFormat(t *testing.T, what testableFormat, format, with string, va
 	// text encoding interface
 	b := []byte(with)
 	err := what.UnmarshalText(b)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	val := reflect.Indirect(reflect.ValueOf(what))
 	strVal := val.String()
 	assert.Equalf(t, with, strVal, "[%s]UnmarshalText: expected %v and %v to be value equal", format, strVal, with)
 
 	b, err = what.MarshalText()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equalf(t, []byte(with), b, "[%s]MarshalText: expected %v and %v to be value equal as []byte", format, string(b), with)
 
 	// Stringer
@@ -367,23 +445,23 @@ func testStringFormat(t *testing.T, what testableFormat, format, with string, va
 	// JSON encoding interface
 	bj := []byte("\"" + with + "\"")
 	err = what.UnmarshalJSON(bj)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	val = reflect.Indirect(reflect.ValueOf(what))
 	strVal = val.String()
 	assert.EqualValuesf(t, with, strVal, "[%s]UnmarshalJSON: expected %v and %v to be value equal", format, strVal, with)
 
 	b, err = what.MarshalJSON()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equalf(t, bj, b, "[%s]MarshalJSON: expected %v and %v to be value equal as []byte", format, string(b), with)
 
 	// bson encoding interface
 	bsonData, err := bson.Marshal(what)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	resetValue(t, format, what)
 
 	err = bson.Unmarshal(bsonData, what)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	val = reflect.Indirect(reflect.ValueOf(what))
 	strVal = val.String()
 	assert.EqualValuesf(t, with, strVal, "[%s]bson.Unmarshal: expected %v and %v to be equal (reset value) ", format, what, with)
@@ -391,23 +469,23 @@ func testStringFormat(t *testing.T, what testableFormat, format, with string, va
 	// Scanner interface
 	resetValue(t, format, what)
 	err = what.Scan(with)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	val = reflect.Indirect(reflect.ValueOf(what))
 	strVal = val.String()
 	assert.EqualValuesf(t, with, strVal, "[%s]Scan: expected %v and %v to be value equal", format, strVal, with)
 
 	err = what.Scan([]byte(with))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	val = reflect.Indirect(reflect.ValueOf(what))
 	strVal = val.String()
 	assert.EqualValuesf(t, with, strVal, "[%s]Scan: expected %v and %v to be value equal", format, strVal, with)
 
 	err = what.Scan(123)
-	assert.Error(t, err)
+	require.Error(t, err)
 
 	// Valuer interface
 	sqlvalue, err := what.Value()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	sqlvalueAsString, ok := sqlvalue.(string)
 	if assert.Truef(t, ok, "[%s]Value: expected driver value to be a string", format) {
 		assert.EqualValuesf(t, with, sqlvalueAsString, "[%s]Value: expected %v and %v to be equal", format, sqlvalue, with)
@@ -427,7 +505,7 @@ func resetValue(t *testing.T, format string, what encoding.TextUnmarshaler) {
 	t.Helper()
 
 	err := what.UnmarshalText([]byte("reset value"))
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	val := reflect.Indirect(reflect.ValueOf(what))
 	strVal := val.String()
 	assert.Equalf(t, "reset value", strVal, "[%s]UnmarshalText: expected %v and %v to be equal (reset value) ", format, strVal, "reset value")
@@ -774,3 +852,48 @@ func TestDeepCopyPassword(t *testing.T) {
 	out3 := inNil.DeepCopy()
 	assert.Nil(t, out3)
 }
+
+func BenchmarkIsUUID(b *testing.B) {
+	const sampleSize = 100
+	rxUUID := regexp.MustCompile(UUIDPattern)
+	rxUUID3 := regexp.MustCompile(UUID3Pattern)
+	rxUUID4 := regexp.MustCompile(UUID4Pattern)
+	rxUUID5 := regexp.MustCompile(UUID5Pattern)
+
+	uuids := make([]string, 0, sampleSize)
+	uuid3s := make([]string, 0, sampleSize)
+	uuid4s := make([]string, 0, sampleSize)
+	uuid5s := make([]string, 0, sampleSize)
+
+	for i := 0; i < sampleSize; i++ {
+		seed := []byte(uuid.Must(uuid.NewRandom()).String())
+		uuids = append(uuids, uuid.Must(uuid.NewRandom()).String())
+		uuid3s = append(uuid3s, uuid.NewMD5(uuid.NameSpaceURL, seed).String())
+		uuid4s = append(uuid4s, uuid.Must(uuid.NewRandom()).String())
+		uuid5s = append(uuid5s, uuid.NewSHA1(uuid.NameSpaceURL, seed).String())
+	}
+
+	b.Run("IsUUID - google.uuid", benchmarkIs(uuids, IsUUID))
+	b.Run("IsUUID - regexp", benchmarkIs(uuids, func(id string) bool { return rxUUID.MatchString(id) }))
+
+	b.Run("IsUUIDv3 - google.uuid", benchmarkIs(uuid3s, IsUUID3))
+	b.Run("IsUUIDv3 - regexp", benchmarkIs(uuid3s, func(id string) bool { return rxUUID3.MatchString(id) }))
+
+	b.Run("IsUUIDv4 - google.uuid", benchmarkIs(uuid4s, IsUUID4))
+	b.Run("IsUUIDv4 - regexp", benchmarkIs(uuid4s, func(id string) bool { return rxUUID4.MatchString(id) }))
+
+	b.Run("IsUUIDv5 - google.uuid", benchmarkIs(uuid5s, IsUUID5))
+	b.Run("IsUUIDv5 - regexp", benchmarkIs(uuid5s, func(id string) bool { return rxUUID5.MatchString(id) }))
+}
+
+func benchmarkIs(input []string, fn func(string) bool) func(*testing.B) {
+	return func(b *testing.B) {
+		var isTrue bool
+		b.ReportAllocs()
+		b.ResetTimer()
+		for i := 0; i < b.N; i++ {
+			isTrue = fn(input[i%len(input)])
+		}
+		fmt.Fprintln(io.Discard, isTrue)
+	}
+}
